/*
  A lot of example of how to run analysis of local or PDC06 produced
  simulation files.

  This file sets up the environment for analysing:

  RunAliRootLocal - local files produced with the same version of AliRoot
  RunRootLocal - locally copied PDC06 files 
  RunAlien - PDC06 files on the grid, either locally (alienroot) or batch. 
  RunCAF - hopefully to come soon. 

  see the functions for more details. ALL THE FUNCTIONS CALLS THE SAME
  FUNCTION FOR THE ACTUAL ANALYSIS: ptspectra

  The files:

  AliAnalysisTaskPt.h
  AliAnalysisTaskPt.cxx

  contains the analysis task which is a dummy task, but shows how to access MC
  input data and reconstructed data at the same time in the analysis
  framework.

  The package files (.par) are all produced with a recent version of AliRoot:

  cd $ALICE_ROOT
  make ANALYSIS.par
  cp /tmp/ANALYSIS.par <some_path>

  EXCEPT for ESD.par which is the patched version for reading PDC06 data sent
  to the alice-project-analysis-task-force@cern.ch mailing list by Andreas
  Morsch on the 14t of August.

  The package files are only used when running with root!

  The following files are for alien analysis:

  collection.xml
  ptspectra.jdl
  root_ptspectra.sh
*/

void ptspectra(TChain* chain, Char_t* analysisPlace);
void RunAliRootLocal(Char_t* esdFileName);
void RunRootLocal(Char_t* esdFileName = "PDC06file/AliESDs.root");
void RunAlien(Char_t* xmlFileName="collection.xml", const Int_t nFiles=5);
void SetupPackagesLocally();
Int_t setupPar(const char* pararchivename);

//________________________________________________________________
void ptspectra(TChain* chain, Char_t* analysisPlace) 
{
    cout << "Compiling task" << endl;
    gROOT->LoadMacro("AliAnalysisTaskPt.cxx++");
  
    // Make the analysis manager and connect the MCTruth
    AliMCEventHandler* mcHandler = new AliMCEventHandler();
    AliAnalysisManager *mgr = 
	new AliAnalysisManager("testAnalysis");
    mgr->SetMCtruthEventHandler(mcHandler);
    // Add Pt task
    AliAnalysisTask *taskPt = new AliAnalysisTaskPt("TaskPt");
    mgr->AddTask(taskPt);
  
    // Create containers for input/output
    AliAnalysisDataContainer* input = 
	mgr->CreateContainer("esdchain", TChain::Class(), 
			     AliAnalysisManager::kInputContainer);
    AliAnalysisDataContainer* ptspectraOut = 
	mgr->CreateContainer("ptspectraOut", TList::Class(),    
			     AliAnalysisManager::kOutputContainer, 
			     "ptspectra.root");
  
    mgr->ConnectInput(taskPt,  0, input);
    mgr->ConnectOutput(taskPt, 0, ptspectraOut);
    input->SetData(chain);
  
    mgr->SetDebugLevel(2);
  
    if (!mgr->InitAnalysis())
	return;
  
    mgr->PrintStatus();
  
    mgr->StartAnalysis(analysisPlace, chain);
}

//________________________________________________________________
void RunAliRootLocal(Char_t* esdFileName)
{
    // Setups the environment to run with the local aliroot
    // on a locally produced file (not PDC06!)
    gSystem->Load("libANALYSIS.so");
    gROOT->SetMacroPath(Form("%s:$ALICE_ROOT/PWG2/AnalysisMacros/Local/",
			     gROOT->GetMacroPath()));
    TChain* chain = new TChain("esdTree");
    chain->Add(esdFileName);
    ptspectra(chain, "local");
}

//________________________________________________________________
void RunRootLocal(Char_t* esdFileName)
{
    // Setups the environment to run with the local root version
    // on a PDC06 produced file (e.g. copied from the grid)
    SetupPackagesLocally();

    TChain* chain = new TChain("esdTree");
    chain->Add(esdFileName);
    ptspectra(chain, "local");
}

//________________________________________________________________
void RunAlien(Char_t* xmlFileName, const Int_t nFiles)
{
    /* Two options: 
       
    1. To run with the local alienroot on a PDC06 produced file stored on the
    grid is straightforward:
    alien-token-init
    alienroot
    .L ptspectra.C
    RunAlien();
    
    2. To run alien batch jobs is more complicated:

    Go to the ptspectra dir and from there start aliensh:
    alien-token-init
    aliensh
    
    Set environment if you are outside CERN:
    export alien_CLOSE_SE=ALICE::CERN::se 
    
    Go to your par directory and copy in the packages:
    cd $HOME/par
    cp file:STEERBase.par STEERBase.par
    cp file:ESD.par ESD.par
    cp file:AOD.par AOD.par
    cp file:ANALYSIS.par ANALYSIS.par

    Go to the bin dir and copy in the executable:
    cd $HOME/bin
    cp file:root_ptspectra.C root_ptspectra.C

    Go to the macro dir and copy in the macros:
    cp file:AliAnalysisTaskPt.h AliAnalysisTaskPt.h
    cp file:AliAnalysisTaskPt.cxx AliAnalysisTaskPt.cxx
    cp file:ptspectra.C ptspectra.C

    Go to the xml dir and copy in the collection:
    cd $HOME/xml
    cp file:collection.xml collection.xml

    Go to the jdl dir and copy in the collection:
    cd $HOME/jdl
    cp file:ptspectra.jdl ptspectra.jdl
    
    In both cases the collection file was made by calling this command in
    aliensh (which unfortunately has a stop commenting sign in it:-):
    */
//    find -l 2 -x collection /alice/sim/2006/pp_minbias/5257/9*/ AliESDs.root > collection.xml
    /*
      
    */
    
    SetupPackagesLocally();

    TGrid::Connect("alien://");

    TAlienCollection * myCollection = 
	new TAlienCollection(xmlFileName, nFiles) ; 
    if (!myCollection) { 
	cout << "XML collection file: " << xmlFileName << " not found" << endl;
	return;
    }

    TChain* chain = new TChain("esdTree");
    
    cout << "Preparing the file list" << endl; 
    myCollection->Reset() ; 
    while ( myCollection->Next() ) {
	cout << "Adding ESD file: " << myCollection->GetTURL("") << endl; 
	chain->Add(myCollection->GetTURL("")) ; 
    }
    
    ptspectra(chain, "alien");
}

//________________________________________________________________
void SetupPackagesLocally()
{
    //____________________________________________________//
    //_____________Setting up STEERBase.par_______________//
    //____________________________________________________//
    setupPar("STEERBase");
    gSystem->Load("STEERBase/libSTEERBase.so");
    
    //____________________________________________________//
    //_____________Setting up ESD.par_____________________//
    //____________________________________________________//
    setupPar("ESD");
    gSystem->Load("ESD/libVMC.so");
    gSystem->Load("ESD/libESD.so");

    //____________________________________________________//
    //_____________Setting up AOD.par_____________________//
    //____________________________________________________//
    setupPar("AOD");
    gSystem->Load("AOD/libAOD.so");
  
    //_________________________________________________________//
    //_____________Setting up ANALYSIS.par_____________________//
    //_________________________________________________________//
    setupPar("ANALYSIS");
    gSystem->Load("ANALYSIS/libANALYSIS.so");

}

//________________________________________________________________
void RunCAF()
{
    // Setups the environment to run
    // on a locally produced file
    // Note that this does not work
    TProof* proof = TProof::Open("pchristi@lxb6046.cern.ch");

    proof->ShowEnabledPackages();
    // Enable the STEERBase Package
    proof->UploadPackage("STEERBase.par");
    proof->EnablePackage("STEERBase");
    // Enable the ESD Package
    proof->UploadPackage("ESD.par");
    proof->EnablePackage("ESD");
    // Enable the AOD Package
    proof->UploadPackage("AOD.par");
    proof->EnablePackage("AOD");
    // Enable the Analysis Package
    proof->UploadPackage("ANALYSIS.par");
    proof->EnablePackage("ANALYSIS");

    gROOT->LoadMacro("CreateESDChain.C");
    TChain* chain = CreateESDChain("ESD100_110_v4.txt");

    ptspectra(chain, "proof");
}


//________________________________________________________________
Int_t setupPar(const char* pararchivename) 
{
    ///////////////////
    // Setup PAR File//
    ///////////////////
    if (pararchivename) {
	char processline[1024];
	sprintf(processline,".! tar xvzf %s.par",pararchivename);
	gROOT->ProcessLine(processline);
	const char* ocwd = gSystem->WorkingDirectory();
	gSystem->ChangeDirectory(pararchivename);
    
	// check for BUILD.sh and execute
	if (!gSystem->AccessPathName("PROOF-INF/BUILD.sh")) {
	    printf("*******************************\n");
	    printf("*** Building PAR archive    ***\n");
	    printf("*******************************\n");

	    if (gSystem->Exec("PROOF-INF/BUILD.sh")) {
		Error("runAnalysis","Cannot Build the PAR Archive! - Abort!");
		return -1;
	    }
	}
	// check for SETUP.C and execute
	if (!gSystem->AccessPathName("PROOF-INF/SETUP.C")) {
	    printf("*******************************\n");
	    printf("*** Setup PAR archive       ***\n");
	    printf("*******************************\n");
	    gROOT->Macro("PROOF-INF/SETUP.C");
	}
    
	gSystem->ChangeDirectory("../");
    }
    return 1;
}

